<head><link href="/css/app.c0d7cdb01c21c4ae6c4e.css" rel="stylesheet"></head><!doctype html>
<title>Spintop</title>

<!-- TODO(Kenadia): sockjs-client should be included via node. -->
<script src="https://cdnjs.cloudflare.com/ajax/libs/sockjs-client/1.1.4/sockjs.min.js"></script>
<link rel="icon" type="image/x-icon" href="img/spintop-icon.png" />

<base href="/">
<htf-app config="{{ json_encode(config) }}">Loading...</htf-app>
<script type="text/javascript" src="/js/polyfills.c0d7cdb01c21c4ae6c4e.js"></script><script type="text/javascript" src="/js/vendor.c0d7cdb01c21c4ae6c4e.js"></script><script type="text/javascript" src="/js/app.c0d7cdb01c21c4ae6c4e.js"></script>